@@ -1,6 +1,7 @@
 import pika, requests, threading, json, time
 from logzero import logger
-
+from datetime import datetime
+from eventemitter import EventEmitter
 
 class AutoSolve:
     SUCCESS = "info"
@@ -19,13 +20,12 @@ class AutoSolve:
         self.access_token = params["access_token"]
         self.api_key = params["api_key"]
         self.client_key = params["client_key"]
-        self.receiver_function = params["receiver_function"]
-        self.cancel_function = params["cancel_function"]
         self.username = ""
         self.vhost = ""
         self.routing_key = self.api_key.replace("-", "")
         self.routing_key_receiver = ""
         self.routing_key_cancel = ""
+        self.emitter = EventEmitter()
         self.receiver_queue_name = "queues.token.response." + self.routing_key
         self.cancel_queue_name = "queues.token.request.cancel." + self.routing_key
         self.connection = None
@@ -35,6 +35,7 @@ class AutoSolve:
         self.debug = params["debug"]
         self.ready = False
         self.connected = False
+        self.should_alert_on_cancel = params["should_alert_on_cancel"]
         self.message_backlog = []
         self.start_connection_process()
 
@@ -103,7 +104,7 @@ class AutoSolve:
 
             self.channel.basic_consume(queue=self.receiver_queue_name, auto_ack=True, on_message_callback=self.on_receiver_message)
 
-            if self.cancel_function is not None:
+            if self.should_alert_on_cancel:
                 self.register_cancel_queue()
 
             self.log("Beginning message consumption", self.SUCCESS)
@@ -128,15 +129,13 @@ class AutoSolve:
 
     def on_receiver_message(self, channel, method_frame, header_frame, body):
         json_string = "".join(chr(x) for x in body)
-        json_object = json.loads(json_string)
         self.log("Message Received: " + json_string, self.SUCCESS)
-        self.receiver_function(json_object)
+        self.emitter.emit("AutoSolveResponse", json_string)
 
     def on_cancel_message(self, channel, method_frame, header_frame, body):
         json_string = "".join(chr(x) for x in body)
-        json_object = json.loads(json_string)
         self.log("Message Received: " + json_string, self.SUCCESS)
-        self.cancel_function(json_object)
+        self.emitter.emit("AutoSolveResponse_Cancel", json_string)
 
     def handle_connection_error(self):
         self.connected = False
@@ -155,6 +154,7 @@ class AutoSolve:
             self.log("Connection attempts unsuccessful", self.ERROR)
 
     def send(self, message, route, exchange):
+        message["createdAt"] = datetime.utcnow()
         message_string = json.dumps(message)
         byte_string = message_string.encode()
         return self.channel.basic_publish(exchange=exchange,
@@ -173,7 +173,7 @@ class AutoSolve:
 
     def cancel_request(self, taskId):
         message = {"taskId" : taskId, "requireResponse" : False}
-        if self.cancel_function is not None:
+        if self.should_alert_on_cancel:
             message["requireResponse"] = True
 
         self.log("Sending cancel message for task: " + message['taskId'], self.STATUS)
